/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.beans.session;

import java.lang.reflect.Modifier;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;
import kieker.analysis.model.analysisMetaModel.MIAnalysisPlugin;

import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.MIReader;
import kieker.analysis.model.analysisMetaModel.impl.MAnalysisMetaModelFactory;
import kieker.analysis.plugin.AbstractAnalysisPlugin;
import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.plugin.AbstractReaderPlugin;
import kieker.webgui.common.FileManager;
import kieker.webgui.common.PluginClassLoader;
import kieker.webgui.common.PluginFinder;

import org.primefaces.event.NodeSelectEvent;
import org.primefaces.model.DefaultTreeNode;
import org.primefaces.model.TreeNode;

/**
 * This bean can be used to store the currently selected project and the current main project for the user within a session.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@SessionScoped
public class SelectedProjectBean {

	/**
	 * This constant is used as the host for the dependencies.
	 */
	private static final String URL_LOCALHOST = "localhost";
	/**
	 * This constant is used as a protocol for the dependencies.
	 */
	private static final String URL_PROTOCOL_FILE = "file";

	/**
	 * The currently selected node of the current user.
	 */
	private TreeNode selectedNode;
	/**
	 * The selected project of the current user.
	 */
	private MIProject selectedProject;
	/**
	 * The main project of the current user.
	 */
	private MIProject mainProject;

	/**
	 * Creates a new instance of this class.
	 */
	public SelectedProjectBean() {
		/*
		 * No code necessary.
		 */
	}

	/**
	 * Delivers the main project of the current user. It can be null.
	 * 
	 * @return The user's main project.
	 */
	public final MIProject getMainProject() {
		return this.mainProject;
	}

	/**
	 * Sets the main project of the current user.
	 * 
	 * @param mainProject
	 *            The new main project of the current user.
	 */
	public final void setMainProject(final MIProject mainProject) {
		this.mainProject = mainProject;
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, "", "New main project: " + mainProject.getName()));
	}

	/**
	 * Delivers the project currently selected by the user. It can be null.
	 * 
	 * @return The user's currently selected project.
	 */
	public final MIProject getSelectedProject() {
		return this.selectedProject;
	}

	/**
	 * Sets the currently selected project within the bean.
	 * 
	 * @param selectedProject
	 *            The selected project.
	 */
	public final void setSelectedProject(final MIProject selectedProject) {
		this.selectedProject = selectedProject;
	}

	/**
	 * Delivers the currently selected node.
	 * 
	 * @return The selected node.
	 */
	public final TreeNode getSelectedNode() {
		return this.selectedNode;
	}

	/**
	 * This event should be triggered once a new node is selected to make sure that the newly selected node is stored within this bean.
	 * 
	 * @param event
	 *            The event that a node has been selected by the user.
	 */
	public final void onNodeSelect(final NodeSelectEvent event) {
		this.setSelectedNode(event.getTreeNode());
	}

	/**
	 * Sets the currently selected node to a new value.
	 * 
	 * @param selectedNode
	 *            The selected node.
	 */
	public final void setSelectedNode(final TreeNode selectedNode) {
		this.selectedNode = selectedNode;
		if (selectedNode != null && selectedNode.getData() instanceof MIProject) {
			this.setSelectedProject((MIProject) selectedNode.getData());
		} else {
			this.setSelectedProject(null);
		}
	}

	/**
	 * Delivers the "font weight" (whether the font is bold or normal) for a given project. This can be especially be used to mark a selected main project.
	 * 
	 * @param project
	 *            The project to be checked.
	 * @return "bold" if the given project is the current main project, "normal" otherwise.
	 */
	public final String getFontWeight(final MIProject project) {
		if (project == this.mainProject) {
			return "bold";
		} else {
			return "normal";
		}
	}

	/**
	 * This method delivers the available reader-plugins for the current main project. The delivered plugins are never abstract.
	 * 
	 * @return A list with all readers.
	 */
	public final List<Class<AbstractReaderPlugin>> getAvailableReaders() {
		final List<Class<AbstractReaderPlugin>> list = new ArrayList<Class<AbstractReaderPlugin>>();

		if (this.mainProject != null) {
			for (final MIDependency lib : this.mainProject.getDependencies()) {
				try {
					PluginClassLoader.getInstance().addURL(
							new URL(SelectedProjectBean.URL_PROTOCOL_FILE, SelectedProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
				} catch (final MalformedURLException ex) {
					// TODO Log exception
				}
				try {
					final List<Class<?>> plugins = PluginFinder.getAllPluginsWithinJar(new URL(SelectedProjectBean.URL_PROTOCOL_FILE,
							SelectedProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
					for (final Class<?> plugin : plugins) {
						if (!Modifier.isAbstract(plugin.getModifiers()) && AbstractReaderPlugin.class.isAssignableFrom(plugin)) {
							list.add((Class<AbstractReaderPlugin>) plugin);
						}
					}
				} catch (final MalformedURLException ex) {
					ex.printStackTrace();
					// TODO Log exception
				}
			}
		}

		return list;
	}

	/**
	 * This method delivers the available filter-plugins for the current main project. The delivered plugins are never abstract.
	 * 
	 * @return A list with all filter.
	 */
	public final List<Class<AbstractAnalysisPlugin>> getAvailableFilters() {
		final List<Class<AbstractAnalysisPlugin>> list = new ArrayList<Class<AbstractAnalysisPlugin>>();

		if (this.mainProject != null) {
			for (final MIDependency lib : this.mainProject.getDependencies()) {
				try {
					PluginClassLoader.getInstance().addURL(
							new URL(SelectedProjectBean.URL_PROTOCOL_FILE, SelectedProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
				} catch (final MalformedURLException ex) {
					// TODO Log exception
				}
				try {
					final List<Class<?>> plugins = PluginFinder.getAllPluginsWithinJar(new URL(SelectedProjectBean.URL_PROTOCOL_FILE,
							SelectedProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
					for (final Class<?> plugin : plugins) {
						if (!Modifier.isAbstract(plugin.getModifiers()) && AbstractAnalysisPlugin.class.isAssignableFrom(plugin)) {
							list.add((Class<AbstractAnalysisPlugin>) plugin);
						}
					}
				} catch (final MalformedURLException ex) {
					ex.printStackTrace();
					// TODO Log exception
				}
			}
		}

		return list;

	}

	/**
	 * Delivers the available plugins.
	 * 
	 * @return A tree with the available plugins.
	 */
	public final TreeNode getAvailablePluginsRoot() {
		final TreeNode root = new DefaultTreeNode("Root", null);

		final TreeNode readerNode = new DefaultTreeNode("default", "Reader", root);
		final TreeNode filterNode = new DefaultTreeNode("default", "AnalysisPlugins", root);

		if (this.mainProject != null) {
			for (final MIDependency lib : this.mainProject.getDependencies()) {
				try {
					PluginClassLoader.getInstance().addURL(new URL(SelectedProjectBean.URL_PROTOCOL_FILE, SelectedProjectBean.URL_LOCALHOST, lib.getFilePath()));
				} catch (final MalformedURLException ex) {
					// TODO Log exception
				}
				try {
					final List<Class<?>> plugins = PluginFinder.getAllPluginsWithinJar(new URL(SelectedProjectBean.URL_PROTOCOL_FILE,
							SelectedProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
					for (final Class<?> plugin : plugins) {
						if (!Modifier.isAbstract(plugin.getModifiers())) {
							if (AbstractReaderPlugin.class.isAssignableFrom(plugin)) {
								new DefaultTreeNode("reader", plugin, readerNode);
							} else {
								if (AbstractAnalysisPlugin.class.isAssignableFrom(plugin)) {
									new DefaultTreeNode("filter", plugin, filterNode);
								}
							}
						}
					}
				} catch (final MalformedURLException ex) {
					ex.printStackTrace();
					// TODO Log exception
				}
			}
		}
		return root;
	}

	public final void addPlugin(final Class<AbstractPlugin> pluginClass) {
        // TODO It seems like it is necessary to completly load all properties, including the available ports in order to work
		final MAnalysisMetaModelFactory factory = new MAnalysisMetaModelFactory();

		if (AbstractReaderPlugin.class.isAssignableFrom(pluginClass)) {
			final MIReader reader = factory.createReader();
			reader.setClassname(pluginClass.getCanonicalName());
			reader.setName(pluginClass.getSimpleName());

			this.mainProject.getPlugins().add(reader);
		} else {
			final MIAnalysisPlugin filter = factory.createAnalysisPlugin();
			filter.setClassname(pluginClass.getCanonicalName());
			filter.setName(pluginClass.getSimpleName());

			this.mainProject.getPlugins().add(filter);
		}
	}
}
